"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createWaitUntilHealthy = void 0;
var _ElasticsearchCatHealth = require("../../operations/ElasticsearchCatHealth");
var _ElasticsearchCatNodes = require("../../operations/ElasticsearchCatNodes");
var _types = require("../../operations/types");
var _UnhealthyClusterError = require("./UnhealthyClusterError");
var _reason = require("./reason");
var _WaitingHealthyClusterAbortedError = require("./WaitingHealthyClusterAbortedError");
const WAITING_TIME_STEP = 10;
class WaitUntilHealthy {
  aborted = false;
  constructor(client, options) {
    this.client = client;
    this.options = options;
    this.catHealth = new _ElasticsearchCatHealth.ElasticsearchCatHealth(this.client);
    this.catNodes = new _ElasticsearchCatNodes.ElasticsearchCatNodes(this.client);
  }
  abort() {
    this.aborted = true;
  }
  /**
   * @throws UnhealthyClusterError
   * @throws WaitingHealthyClusterAbortedError
   */
  async wait(options) {
    if (this.aborted) {
      throw new _WaitingHealthyClusterAbortedError.WaitingHealthyClusterAbortedError(`Waiting for the cluster to become healthy was aborted even before it started.`);
    }
    const startedAt = new Date();
    const mustEndAt = new Date(startedAt.getTime() + this.options.maxWaitingTime * 1000);
    const waitingTimeStep = this.options.waitingTimeStep || WAITING_TIME_STEP;
    let runs = 1;
    let waitingReason;
    while (waitingReason = await this.shouldWait()) {
      if (new Date() >= mustEndAt) {
        if (options?.onTimeout) {
          await options.onTimeout({
            startedAt,
            mustEndAt,
            waitingTimeStep,
            waitingReason,
            runs
          });
        }
        throw new _UnhealthyClusterError.UnhealthyClusterError(this.options.maxWaitingTime);
      } else if (options?.onUnhealthy) {
        await options.onUnhealthy({
          startedAt,
          mustEndAt,
          waitingTimeStep,
          waitingReason,
          runs
        });
      }
      /**
       * Abort check is separated from other IFs because it can be aborted in onUnhealthy callback.
       */
      if (this.aborted) {
        throw new _WaitingHealthyClusterAbortedError.WaitingHealthyClusterAbortedError();
      }
      runs++;
      await new Promise(resolve => {
        setTimeout(resolve, waitingTimeStep * 1000);
      });
    }
    const runningTime = new Date().getTime() - startedAt.getTime();
    return {
      runningTime,
      runs
    };
  }
  async shouldWait() {
    let health;
    let nodes;
    try {
      health = await this.catHealth.getHealth();
    } catch (ex) {
      return (0, _reason.createClusterHealthStatusReason)({
        description: ex.message,
        minimum: this.options.minClusterHealthStatus,
        current: _types.ElasticsearchCatClusterHealthStatus.Red
      });
    }
    try {
      nodes = await this.catNodes.getNodes();
    } catch (ex) {
      return (0, _reason.createClusterHealthStatusReason)({
        description: ex.message,
        minimum: this.options.minClusterHealthStatus,
        current: _types.ElasticsearchCatClusterHealthStatus.Red
      });
    }
    const clusterHealthStatus = this.transformClusterHealthStatus(health.status);
    const minClusterHealthStatus = this.transformClusterHealthStatus(this.options.minClusterHealthStatus);
    if (clusterHealthStatus > minClusterHealthStatus) {
      return (0, _reason.createClusterHealthStatusReason)({
        minimum: this.options.minClusterHealthStatus,
        current: health.status
      });
    }
    const processorPercent = this.getProcessorPercent(nodes);
    if (processorPercent > this.options.maxProcessorPercent) {
      return (0, _reason.createProcessorReason)({
        maximum: this.options.maxProcessorPercent,
        current: processorPercent
      });
    }
    /**
     * Possibly no max ram definition?
     */
    if (this.options.maxRamPercent === undefined) {
      return false;
    }
    const ramPercent = this.getRamPercent(nodes);
    if (ramPercent > this.options.maxRamPercent) {
      return (0, _reason.createMemoryReason)({
        maximum: this.options.maxRamPercent,
        current: ramPercent
      });
    }
    return false;
  }
  getProcessorPercent(nodes) {
    const total = nodes.reduce((total, node) => {
      return total + parseFloat(node.cpu);
    }, 0);
    return total / nodes.length;
  }
  getRamPercent(nodes) {
    const total = nodes.reduce((total, node) => {
      return total + parseFloat(node["ram.percent"]);
    }, 0);
    return total / nodes.length;
  }
  transformClusterHealthStatus(status) {
    switch (status) {
      case _types.ElasticsearchCatClusterHealthStatus.Green:
        return 1;
      case _types.ElasticsearchCatClusterHealthStatus.Yellow:
        return 2;
      case _types.ElasticsearchCatClusterHealthStatus.Red:
        return 3;
      default:
        return 99;
    }
  }
}
const createWaitUntilHealthy = (client, params) => {
  return new WaitUntilHealthy(client, params);
};
exports.createWaitUntilHealthy = createWaitUntilHealthy;

//# sourceMappingURL=WaitUntilHealthy.js.map